/*-
 * Copyright (c) 2006, 2007 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      VMDumper.cpp
 *
 * $CTPP$
 */
#include <VMDumper.hpp>

#include <StaticData.hpp>
#include <StaticText.hpp>
#include <VMInstruction.hpp>

#include <stdlib.h>
#include <string.h>

#include <stdio.h>
namespace CTPP // C++ Template Engine
{

//
// Constructor
//
VMDumper::VMDumper(const VMMemoryCore & oMemoryCore)
{
	INT_32 iSyscallsDataSize   = 0;
	if (oMemoryCore.syscalls.iUsedDataOffsetsSize == 0) { iSyscallsDataSize = 0; }
	else
	{
		const TextDataIndex & oTMP = oMemoryCore.syscalls.aDataOffsets[oMemoryCore.syscalls.iUsedDataOffsetsSize - 1];
		iSyscallsDataSize = oTMP.offset + oTMP.length + 1;
	}

	INT_32 iStaticTextDataSize = 0;
	if (oMemoryCore.static_text.iUsedDataOffsetsSize == 0) { iStaticTextDataSize = 0; }
	else
	{
		const TextDataIndex & oTMP = oMemoryCore.static_text.aDataOffsets[oMemoryCore.static_text.iUsedDataOffsetsSize - 1];
		iStaticTextDataSize    = oTMP.offset + oTMP.length + 1;
	}

	                    // Header
	iVMExecutableSize = sizeof(VMExecutable) +

	                    // Code segment
	                    sizeof(VMInstruction) * oMemoryCore.code_size +

	                    // Static syscalls segment
	                    iSyscallsDataSize +
	                    // Static syscalls index segment
	                    sizeof(TextDataIndex) * oMemoryCore.syscalls.iUsedDataOffsetsSize +

	                    // Static Data segment
	                    sizeof(StaticDataVar) * oMemoryCore.static_data.iUsedDataSize +

	                    // Static Text segment
	                    iStaticTextDataSize +
	                    // Static Text index segment
	                    sizeof(TextDataIndex) * oMemoryCore.static_text.iUsedDataOffsetsSize;

	CHAR_P vRawData = (CHAR_P)malloc(iVMExecutableSize);
	// Make valgind happy
	memset(vRawData, '-', iVMExecutableSize);

	oVMExecutable = (VMExecutable *)vRawData;

	oVMExecutable -> magic[0] = 'C';
	oVMExecutable -> magic[1] = 'T';
	oVMExecutable -> magic[2] = 'P';
	oVMExecutable -> magic[3] = 'P';

	for(UINT_32 iI = 0; iI < 8; ++iI) { oVMExecutable -> version[iI] = 0; }

	oVMExecutable -> entry_point              = 0;
	oVMExecutable -> code_offset              = sizeof(VMExecutable);
	oVMExecutable -> code_size                = sizeof(VMInstruction) * oMemoryCore.code_size;

	// Syscalls segment
	oVMExecutable -> syscalls_offset          = oVMExecutable -> code_offset + oVMExecutable -> code_size;
	oVMExecutable -> syscalls_data_size       = iSyscallsDataSize;

	oVMExecutable -> syscalls_index_offset    = oVMExecutable -> syscalls_offset + oVMExecutable -> syscalls_data_size;
	oVMExecutable -> syscalls_index_size      = sizeof(TextDataIndex) * oMemoryCore.syscalls.iUsedDataOffsetsSize;

	// Data segment
	oVMExecutable -> static_data_offset       = oVMExecutable -> syscalls_index_offset + oVMExecutable -> syscalls_index_size;
	oVMExecutable -> static_data_data_size    = sizeof(StaticDataVar) * oMemoryCore.static_data.iUsedDataSize;

	// Text segment
	oVMExecutable -> static_text_offset       = oVMExecutable -> static_data_offset + oVMExecutable -> static_data_data_size;
	oVMExecutable -> static_text_data_size    = iStaticTextDataSize;

	oVMExecutable -> static_text_index_offset = oVMExecutable -> static_text_offset + oVMExecutable -> static_text_data_size;
	oVMExecutable -> static_text_index_size   = sizeof(TextDataIndex) * oMemoryCore.static_text.iUsedDataOffsetsSize;

	// Copy code segment
	memcpy(vRawData + sizeof(VMExecutable),                              oMemoryCore.instructions,            oVMExecutable -> code_size);

	// Copy syscalls, if need
	if (oVMExecutable -> syscalls_data_size != 0)
	{
		memcpy(vRawData + oVMExecutable -> syscalls_offset,          oMemoryCore.syscalls.sData,          oVMExecutable -> syscalls_data_size);
		// Copy static text index segment
		memcpy(vRawData + oVMExecutable -> syscalls_index_offset,    oMemoryCore.syscalls.aDataOffsets,   oVMExecutable -> syscalls_index_size);
	}

	// Copy static data segment, if need
	if (oVMExecutable -> static_data_data_size != 0)
	{
		// Copy static data segment
		memcpy(vRawData + oVMExecutable -> static_data_offset,       oMemoryCore.static_data.aData,        oVMExecutable -> static_data_data_size);
	}

	// Copy static text segment, if need
	if (oVMExecutable -> static_text_data_size != 0)
	{
		memcpy(vRawData + oVMExecutable -> static_text_offset,       oMemoryCore.static_text.sData,        oVMExecutable -> static_text_data_size);
		// Copy static text index segment
		memcpy(vRawData + oVMExecutable -> static_text_index_offset, oMemoryCore.static_text.aDataOffsets, oVMExecutable -> static_text_index_size);
	}
}

//
// Constructor
//
VMDumper::VMDumper(const UINT_32        & iCodeSize,
                   const VMInstruction  * aInstructions,
                   const StaticText     & oSyscalls,
                   const StaticData     & oStaticData,
                   const StaticText     & oStaticText)
{
	                    // Header
	iVMExecutableSize = sizeof(VMExecutable) +

	                    // Code segment
	                    sizeof(VMInstruction) * iCodeSize +

	                    // Static syscalls segment
	                    oSyscalls.iUsedDataSize +
	                    // Static syscalls index segment
	                    sizeof(TextDataIndex) * oSyscalls.iUsedDataOffsetsSize +

	                    // Static Data segment
	                    sizeof(StaticDataVar) * oStaticData.iUsedDataSize +

	                    // Static Text segment
	                    oStaticText.iUsedDataSize +
	                    // Static Text index segment
	                    sizeof(TextDataIndex) * oStaticText.iUsedDataOffsetsSize;

	CHAR_P vRawData = (CHAR_P)malloc(iVMExecutableSize);
	// Make valgind happy
	memset(vRawData, '-', iVMExecutableSize);

	oVMExecutable = (VMExecutable *)vRawData;

	oVMExecutable -> magic[0] = 'C';
	oVMExecutable -> magic[1] = 'T';
	oVMExecutable -> magic[2] = 'P';
	oVMExecutable -> magic[3] = 'P';

	for(UINT_32 iI = 0; iI < 8; ++iI) { oVMExecutable -> version[iI] = 0; }

	oVMExecutable -> entry_point              = 0;
	oVMExecutable -> code_offset              = sizeof(VMExecutable);
	oVMExecutable -> code_size                = sizeof(VMInstruction) * iCodeSize;

	// Syscalls segment
	oVMExecutable -> syscalls_offset          = oVMExecutable -> code_offset + oVMExecutable -> code_size;
	oVMExecutable -> syscalls_data_size       = oSyscalls.iUsedDataSize;

	oVMExecutable -> syscalls_index_offset    = oVMExecutable -> syscalls_offset + oVMExecutable -> syscalls_data_size;
	oVMExecutable -> syscalls_index_size      = sizeof(TextDataIndex) * oSyscalls.iUsedDataOffsetsSize;

	// Data segment
	oVMExecutable -> static_data_offset       = oVMExecutable -> syscalls_index_offset + oVMExecutable -> syscalls_index_size;
	oVMExecutable -> static_data_data_size    = sizeof(StaticDataVar) * oStaticData.iUsedDataSize;

	// Text segment
	oVMExecutable -> static_text_offset       = oVMExecutable -> static_data_offset + oVMExecutable -> static_data_data_size;
	oVMExecutable -> static_text_data_size    = oStaticText.iUsedDataSize;

	oVMExecutable -> static_text_index_offset = oVMExecutable -> static_text_offset + oVMExecutable -> static_text_data_size;
	oVMExecutable -> static_text_index_size   = sizeof(TextDataIndex) * oStaticText.iUsedDataOffsetsSize;

	// Copy code segment
	memcpy(vRawData + sizeof(VMExecutable),                              aInstructions,            oVMExecutable -> code_size);

	// Copy syscalls, if need
	if (oVMExecutable -> syscalls_data_size != 0)
	{
		memcpy(vRawData + oVMExecutable -> syscalls_offset,          oSyscalls.sData,          oVMExecutable -> syscalls_data_size);
		// Copy static text index segment
		memcpy(vRawData + oVMExecutable -> syscalls_index_offset,    oSyscalls.aDataOffsets,   oVMExecutable -> syscalls_index_size);
	}

	// Copy static data segment, if need
	if (oVMExecutable -> static_data_data_size != 0)
	{
		// Copy static data segment
		memcpy(vRawData + oVMExecutable -> static_data_offset,       oStaticData.aData,        oVMExecutable -> static_data_data_size);
	}

	// Copy static text segment, if need
	if (oVMExecutable -> static_text_data_size != 0)
	{
		memcpy(vRawData + oVMExecutable -> static_text_offset,       oStaticText.sData,        oVMExecutable -> static_text_data_size);
		// Copy static text index segment
		memcpy(vRawData + oVMExecutable -> static_text_index_offset, oStaticText.aDataOffsets, oVMExecutable -> static_text_index_size);
	}
}

//
// Get constructed executable
//
const VMExecutable * VMDumper::GetExecutable(UINT_32 & iExecutableSize)
{
	iExecutableSize = iVMExecutableSize;

return oVMExecutable;
}

//
// A destructor
//
VMDumper::~VMDumper()
{
	free(oVMExecutable);
}

} // namespace CTPP
// End.
