/*-
 * Copyright (c) 2006, 2007 CTPP Team
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 4. Neither the name of the CTPP Team nor the names of its contributors
 *    may be used to endorse or promote products derived from this software
 *    without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *      VMFileLoader.cpp
 *
 * $CTPP$
 */

#include <VMFileLoader.hpp>

#include <CTPPException.hpp>
#include <VMExecutable.hpp>
#include <VMMemoryCore.hpp>

#include <sys/types.h>
#include <sys/stat.h>

#include <errno.h>
#include <stdio.h>
#include <stdlib.h>

namespace CTPP // C++ Template Engine
{

//
// Constructor
//
VMFileLoader::VMFileLoader(CCHAR_P szFileName)
{
	// Get file size
	struct stat oStat;
	if (stat(szFileName, &oStat) == -1) { throw CTPPUnixException("stat", errno); }
	if (oStat.st_size == 0) { throw CTPPLogicError("Cannot get size of file"); }

	// Load file
	FILE * F = fopen(szFileName, "r");
	if (F == NULL) { throw CTPPUnixException("fopen", errno); }

	// Allocate memory
	oCore = (VMExecutable *)malloc(oStat.st_size);
	// Read from file
	fread(oCore, oStat.st_size, 1, F);
	// All Done
	fclose(F);

	if (oCore -> magic[0] == 'C' &&
	    oCore -> magic[1] == 'T' &&
	    oCore -> magic[2] == 'P' &&
	    oCore -> magic[3] == 'P')
	{
		pVMMemoryCore = new VMMemoryCore(oCore);
	}
	else
	{
		free(oCore);
		throw CTPPLogicError("Not an CTPP bytecode file.");
	}
}

//
// Get ready-to-run program
//
const VMMemoryCore * VMFileLoader::GetCore() const { return pVMMemoryCore; }

//
// A destructor
//
VMFileLoader::~VMFileLoader() throw()
{
	delete pVMMemoryCore;
	free(oCore);
}

} // namespace CTPP
// End.
